function varargout = moq(varargin)
% MOQ Application M-file for moq.fig
%    FIG = MOQ launch moq GUI.
%    MOQ('callback_name', ...) invoke the named callback.
%
% MOQ is a friendly interface for measuring fundamental frequency and open quotient on
% EGG and DEGG signals, using one of the following method :
% - DEGG DECOM (from Henrich et al., 2004)
% - EGG 35% and EGG 50% (from Rothenberg and Mahshie, 1988)
% The result is a structure rsl containing information on :
% - time (t)
% - vocal intensity (Idb)
% - fundamental frequency (f0)
% - open quotient (Oq)
% - number of opening and closing peaks within a glottal period (in the case of DECOM method)
% 
% Author : N. Henrich, LAM, Dec. 2002
% Modification 25/02/2007


% Last Modified by GUIDE v2.0 10-Dec-2002 15:57:26
homepath = pwd;
%addpath([homepath '\sf'])
%addpath([homepath '\AUDIO'])

if nargin == 0  % LAUNCH GUI
    
    fig = openfig(mfilename,'reuse');
    set(fig,'Name','MOQ interface')
    % Generate a structure of handles to pass to callbacks, and store it. 
    handles = guihandles(fig);
    handles.butcol = [0.93 0.94 0.73];
    handles.phrase = [];
    handles.egg = [];
    handles.degg = [];
    
    handles.Fe = [];
    handles.rsl.Oq = [];
    handles.rsl.f0 = [];
    handles.rsl.t  = [];
    guidata(fig, handles);
    
    
    if nargout > 0
        varargout{1} = fig;
    end
    
elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK
    
    try
        if (nargout)
            [varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
        else
            feval(varargin{:}); % FEVAL switchyard
        end
    catch
        disp(lasterr);
    end
    
end


%| ABOUT CALLBACKS:
%| GUIDE automatically appends subfunction prototypes to this file, and 
%| sets objects' callback properties to call them through the FEVAL 
%| switchyard above. This comment describes that mechanism.
%|
%| Each callback subfunction declaration has the following form:
%| <SUBFUNCTION_NAME>(H, EVENTDATA, HANDLES, VARARGIN)
%|
%| The subfunction name is composed using the object's Tag and the 
%| callback type separated by '_', e.g. 'slider2_Callback',
%| 'figure1_CloseRequestFcn', 'axis1_ButtondownFcn'.
%|
%| H is the callback object's handle (obtained using GCBO).
%|
%| EVENTDATA is empty, but reserved for future use.
%|
%| HANDLES is a structure containing handles of components in GUI using
%| tags as fieldnames, e.g. handles.figure1, handles.slider2. This
%| structure is created at GUI startup using GUIHANDLES and stored in
%| the figure's application data using GUIDATA. A copy of the structure
%| is passed to each callback.  You can store additional information in
%| this structure at GUI startup, and you can change the structure
%| during callbacks.  Call guidata(h, handles) after changing your
%| copy to replace the stored original so that subsequent callbacks see
%| the updates. Type "help guihandles" and "help guidata" for more
%| information.
%|
%| VARARGIN contains any extra arguments you have passed to the
%| callback. Specify the extra arguments by editing the callback
%| property in the inspector. By default, GUIDE sets the property to:
%| <MFILENAME>('<SUBFUNCTION_NAME>', gcbo, [], guidata(gcbo))
%| Add any extra arguments after the last argument, before the final
%| closing parenthesis.
% --------------------------------------------------------------------
function varargout = audiofile_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = EGGfile_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = openfile_Callback(h, eventdata, handles, varargin)

% current path
homepath = pwd;

% get an audio file
[audiofile,audiopathfile] = uigetfile('*.*','Please choose the audio file to be downloaded');

% back to current path
cd(homepath)

% save datas in the structure "handles"
set(handles.audiofile,'String',audiofile)
handles.audiopathfile = audiopathfile;

% automatic EGG file detection - in the case where EGG = audio + ext 'egg'
[pathstr,audiofilename,audiofileext,versn] = fileparts([audiopathfile audiofile]);
set(handles.EGGfile,'String',[audiofilename '.egg'])
handles.EGGpathfile = audiopathfile;

% SAVE DATAS 
guidata(gcbo,handles);

% --------------------------------------------------------------------
function varargout = openEGG_Callback(h, eventdata, handles, varargin)

% current path
homepath = pwd;

% get an EGG file
[EGGfile,EGGpathfile]=uigetfile('*.*','Please choose the EGG file to be downloaded');

% back to current path
cd(homepath)

% save datas in the structure "handles"
[pathstr,EGGfilename,EGGfileext,versn] = fileparts([EGGpathfile EGGfile]);

set(handles.EGGfile,'String',EGGfile)
handles.EGGpathfile = EGGpathfile;

% SAVE DATAS 
guidata(gcbo,handles);

% --------------------------------------------------------------------
function varargout = downloadfile_Callback(h, eventdata, handles, varargin)

% INFORMATIONS ABOUT THE FILES (FROM  THE AUDIO FILE)
[pathstr,audiofilename,audiofileext,versn] = fileparts([handles.audiopathfile get(handles.audiofile,'String')]);

% THE MAT FILE WILL BE NAMED AS THE AUDIO FILE
handles.filename = audiofilename;

% DOWNLOAD WAV FILE
[phrase,Fe_audio,nbits] = wavread([handles.audiopathfile get(handles.audiofile,'String')]);
[egg,Fe_egg,nbits] = wavread([handles.EGGpathfile get(handles.EGGfile,'String')]);

set(handles.ffe,'String',num2str(Fe_audio));
set(handles.fduration,'String',num2str(length(phrase)/Fe_audio));

if Fe_audio == Fe_egg
    Fe = Fe_audio;
else
    error('Audio and EGG files do not have the same sampling frequency ...');
end

handles.phrase = phrase;
handles.egg = egg;
handles.Fe = Fe;
guidata(gcbo,handles);


% --------------------------------------------------------------------
function varargout = methode_analyse_Callback(h, eventdata, handles, varargin)    


% --------------------------------------------------------------------
function varargout = fformat_Callback(h, eventdata, handles, varargin)
% --------------------------------------------------------------------
function varargout = ffe_Callback(h, eventdata, handles, varargin)

% --------------------------------------------------------------------
function varargout = fduration_Callback(h, eventdata, handles, varargin)




% --------------------------------------------------------------------
function varargout = Untitled_1_Callback(h, eventdata, handles, varargin)




% --------------------------------------------------------------------
function varargout = tframe_Callback(h, eventdata, handles, varargin)

tframe = get(handles.tframe,'String');
eval(sprintf('set(handles.tframe,''Value'',%s)',tframe))




% --------------------------------------------------------------------
function varargout = tstep_Callback(h, eventdata, handles, varargin)

tstep = get(handles.tstep,'String');
eval(sprintf('set(handles.tstep,''Value'',%s)',tstep))



% --------------------------------------------------------------------
function varargout = analyse_Callback(h, eventdata, handles, varargin)

set(handles.analyse,'BackgroundColor','r');
pause(0.1)

moq_analyse(h, eventdata, handles, varargin);
set(handles.analyse,'BackgroundColor',handles.butcol);


% --------------------------------------------------------------------
function varargout = save_Callback(h, eventdata, handles, varargin)

set(handles.save,'BackgroundColor','r');
pause(0.1)

audiofile = get(handles.audiofile,'String');
[pathstr,filename,ext,versn] = fileparts([handles.audiopathfile audiofile]);

f0 = handles.rsl.f0;
t  = handles.rsl.t;
Oq = handles.rsl.Oq;
Idb = handles.rsl.Idb;

handles.rsl

phrase = handles.phrase;
egg = handles.egg;
degg = handles.degg;
Fe = handles.Fe;

analysis = [];
analysis.tstep = get(handles.tstep,'Value');
analysis.tframe = get(handles.tframe,'Value');

[newfile,newpath] = uiputfile([filename '.mat'],'Save matlab file name');

oldpath = pwd;
cd(newpath)
 
if isfield(handles.rsl,'npicf')
    npicf = handles.rsl.npicf;
    npico = handles.rsl.npico;
    save(newfile,'phrase','egg','degg','Fe','t','Oq','f0','Idb','analysis','npicf','npico');
else
   save(newfile,'phrase','egg','degg','Fe','t','Oq','f0','Idb','analysis'); 
end

cd(oldpath)

set(handles.save,'BackgroundColor',handles.butcol);





% --------------------------------------------------------------------
function varargout = figures_Callback(h, eventdata, handles, varargin)

set(handles.figures,'BackgroundColor','r');
pause(0.1)
% [pathstr,filename,ext,versn] = fileparts([handles.dirfile get(handles.filename,'String')]);

if isfield(handles,'filename')
    
    filename = handles.filename;
    
    f0 = handles.rsl.f0;
    t  = handles.rsl.t;
    Oq = handles.rsl.Oq;
    
    Idb = handles.rsl.Idb;
    
    phrase = handles.phrase;
    egg = handles.egg;
    degg = handles.degg;
    Fe = handles.Fe;
    
else
    % get DATA file
    [DATAfile,DATApathfile]=uigetfile('*.*','Please choose the DATA file to be downloaded (.mat)');
    load([DATApathfile DATAfile])
end


figure
%
% rchantillonnage  16000 Hz
%
phrase16=[];
phrase16=resample(phrase,16000,Fe);
%
% spectrogramme
%
[spectre,fs,ts]=specgram(phrase16,512,16000,512,400);

h1=subplot(4,1,1);
imagesc(ts,fs,dB(spectre))
axis xy;
colormap(WandB(33))
axis([min(t) max(t)  0 8000])
y=ylabel('signal audio');
set(y,'FontSize',12,'FontWeight','bold')
posi = get(y,'Position');
nx = -0.12*max(t);
set(y,'Position',[nx posi(2)+400 posi(3)])

tt = title(filename);
set(tt,'FontSize',16,'FontWeight','bold')
set(h1,'XTicklabel',[])

h2=subplot(4,1,2);

p=plot(t,Idb);
axis([min(t) max(t) min(Idb)-2 max(Idb)+2])
y=ylabel('I (dB)');
set(y,'FontSize',12,'FontWeight','bold')
posi = get(y,'Position');
set(y,'Position',[nx posi(2) posi(3)])
set(p,'Color',[0.792 0.208 0.616])
set(h2,'XTicklabel',[],'XGrid','on','YGrid','on')


h3=subplot(4,1,4);
p=plot(t,f0,'*');
f0ind = find(f0~=0);

if ~isempty(f0ind)
    axis([min(t) max(t) 100 max(f0(f0ind))+50])
else
    axis([min(t) max(t) min(f0)-10 max(f0)+20])
end

y=ylabel('F0');
set(y,'Rotation',0)
set(y,'FontSize',12,'FontWeight','bold')
posi = get(y,'Position');
set(y,'Position',[nx posi(2) posi(3)])
set(p,'Color',[0 0.5 0.5],'MarkerSize',2)
set(h3,'XTicklabel',[],'YGrid','on')


h4=subplot(4,1,3);

p=plot(t,Oq,'*');
axis([min(t) max(t) 0.2 1])
y=ylabel('Oq');
set(y,'Rotation',0)
set(y,'FontSize',12,'FontWeight','bold')
posi = get(y,'Position');
set(y,'Position',[nx posi(2) posi(3)])
set(p,'Color','b','MarkerSize',2)
set(h4,'YGrid','on','YTick',[0:0.1:1])

set(handles.figures,'BackgroundColor',handles.butcol);



% --------------------------------------------------------------------
function varargout = calibIdb_Callback(h, eventdata, handles, varargin)

CAL = get(handles.calibIdb,'String');
eval(sprintf('set(handles.calibIdb,''Value'',%s)',CAL))
IdbCAL = get(handles.calibIdb,'Value');
set(handles.calibIdb,'Value',IdbCAL)


% --------------------------------------------------------------------
function varargout = soundsc_Callback(h, eventdata, handles, varargin)

soundsc(handles.phrase,handles.Fe)

% --------------------------------------------------------------------
function varargout = plotandplay_Callback(h, eventdata, handles, varargin)

plotandplay(handles.phrase,handles.Fe)

% --------------------------------------------------------------------
function varargout = visuEGG_Callback(h, eventdata, handles, varargin)

figure
egg = handles.egg;
L = length(egg);

% by convention, EGG proportional to Vocal Fold Contact Area (VFCA)
EGGconvention = get(handles.EGGconvention,'Value');
if EGGconvention == 2 % IVFCA
    egg = -egg;
end

degg=diff(egg);
degg(L)=degg(L-1);

Fe = handles.Fe;
plot((0:L-1)/Fe,degg/max(degg),(0:L-1)/Fe,egg/max(egg)+1-min(min(egg/max(egg)),0)) 


