function varargout = moq_analyse(h, eventdata, handles, varargin)

% METHOD CHOICE
methodstr = get(handles.methode_analyse,'String');
methodno = get(handles.methode_analyse,'Value');
method = methodstr(methodno);

phrase = handles.phrase;
egg = handles.egg;
Fe = handles.Fe;

% VALUE FOR CALIBRATION OF VOCAL INTENSITY
IdbCAL = get(handles.calibIdb,'Value');


% by convention, EGG proportional to Vocal Fold Contact Area (VFCA)
EGGconvention = get(handles.EGGconvention,'Value');
if EGGconvention == 2 % IVFCA
    egg = -egg;
end


if length(phrase) == length(egg)
    L = length(phrase);
else
    L = length(egg);
    warning('Les fichiers audio et EGG n''ont pas la mme taille')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCULATION OF THE DERIVATIVE (DEGG SIGNAL)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L = length(egg);
degg=diff(egg);
degg(L)=degg(L-1);


%%%%%%%%%%%%%%%%%%%%%%
% ANALYSIS PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%
largfen0 = get(handles.tframe,'Value')/1000*handles.Fe; % window length
tps = get(handles.tstep,'Value')/1000; % step
pas0 = fix(tps*Fe); % step in samples

%%%%%%%%%%%%%%%%%
% INITIALISATION 
%%%%%%%%%%%%%%%%%
k = 1;
N = 0;
largfen = largfen0;

f0min = 80;
f0max = 1500;
disp('Open quotient measurement on the EGG and DEGG signals ...')


% WAITBAR
hwaitbar = waitbar(0,['EGG ANALYSIS USING METHOD :  "' char(method) '"'],'CreateCancelBtn','closereq');


while (N+largfen) <= L
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % analyse du signal audio
    %%%%%%%%%%%%%%%%%%%%%%%%%
    
    % portion de signal  analyser
    % l'analyse en intensit se fait sur un intervalle de 125 ms 
    % (norme CEI 651 / NF EN 60651 / NF S 31-009 sur les sonomtres)
    snd = [];
    fen_snd = fix(0.125*Fe);
    snd = phrase(max(1,N+1-fix(fen_snd/2)):min(N+fix(fen_snd/2),L))';
    
    % signal de pression (en Pa)
    snd = snd/IdbCAL;
    
    % intensit sonore au niveau du micro (en dB) 
    Idb(k) = 10*log10(1/length(snd)*sum((snd.*snd)/(2*10^(-5))^2));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % analyse du signal DEGG
    %%%%%%%%%%%%%%%%%%%%%%%%%
    
    % portion de signal  analyser
    s=[];
    
    switch char(method)

        case 'DEGG DECOM'
            sd = degg(N+1:N+largfen)';

            % mesure de f0 et Oq sur le signal DEGG
            [Oq(k),f0(k),npicf(k),npico(k)] = decom(sd,Fe,[f0min,f0max]);

        case 'DEGG level'

            close(hwaitbar);
            error('Cette mthode n''est pas encore implmente dans MOQ ...');
            
        case 'Howard 3:7'
            s=egg(N+1:N+largfen)';
            [Oq(k),f0(k)] = oq_egg(s,Fe,'howard');

        case 'EGG 35 %'

            s=egg(N+1:N+largfen)';
            [Oq(k),f0(k)] = oq_egg(s,Fe,'35');

        case 'EGG 50 %'

            s=egg(N+1:N+largfen)';
            [Oq(k),f0(k)] = oq_egg(s,Fe,'50');

    end

    %    f0(k)
    %    figure; plot(sd);waitforbuttonpress
    % time vector
    t(k) = (N+largfen) / Fe;
    % waitbar
    waitbar(t(k)/L*Fe,hwaitbar);
    
    % calcul de la largeur de la fentre suivante
    % et du pas d'avancement
    % si f0 n'est pas nulle, on prend 4 priodes
    % et on avance de deux priodes
    if (f0(k) ~= 0)
        largfen = fix(4*Fe/f0(k));
        pas = fix(2*Fe/f0(k));
    else
        largfen = largfen0 ; 
        pas = pas0 ;
    end
    
    
    k=k+1;
    N = N + pas ; 
    
    
end

handles.rsl.Idb = Idb;
handles.rsl.Oq = Oq;
handles.rsl.f0 = f0;
handles.rsl.t  = t;

if strcmp(method,'DEGG DECOM')
    handles.rsl.npicf = npicf;
    handles.rsl.npico = npico;
end

guidata(gcbo,handles);

close(hwaitbar)
